//#include "wgOleDefs.h"
//#include <Definitions.h>
#include <OAIDL.H>
#include <string>
#ifndef _WG_SAFE_ARRAY
#define _WG_SAFE_ARRAY



///
/// wgSafeArray is intended to simplify the use of SAFEARRAY's and provide suitabile 
/// and sufficient functionality to be useful in contexts such as Ole. 
/// It allows easy building of arrays of all and varying types. 
/// It ensures safe cleanup of resources after use. 
/// A SafeArray is a Multi-Vector Array. One can use it like a multi-dimensional array.
/// It allows, in theory, an unlimited number of dimensions this is done through the
/// use of variable arguments. Use the Initialise function to specify the number of dimensions, 
/// and size of each dimension. i.e. To specify a two dimensional array as [5,5]:<br><br>
/// obj.Initialise(vttype, 5, 5, 0);<br><br>
/// where the last argument '0' specifies simply an end to the arguments. This is required as 
/// Initialise uses the 'variable argument' type argument line.<br><br>
/// NOTE: Excel returns, and expects 2 dimensional SAFEARRAY's always, even if you are only 
/// using one dimension, i.e. one column. So! if you want set five rows in one column of an 
/// excel sheet, you would initialise your array like this first:<br><br>
/// obj.Initialise(VT_VARIANT, 5, 1, 0); // rows=5, column=1
///

 
class wgSafeArray : public VARIANT{

	public:

			/// Main constructor
		wgSafeArray();

			/// Copies a VARIANT into this memory space
			/// \param varSafeArray is the VARIANT to be copied into this memory space
		wgSafeArray(const VARIANT& varSafeArray);

			/// Copies the handle to this object so that it can be used in this context. 
			/// If you don't want this object cleaning up this type after it reaches the end 
			/// of it's scope, use the function void ZeroWithoutRelease(); to let it go. 
		wgSafeArray(SAFEARRAY* pSafeArrayBaseType);

			/// Copy constructor
			/// \param other is the wgSafeArray to be copied into this wgSafeArray
		wgSafeArray(const wgSafeArray* other) throw();

			/// Constructor: One Dimension Vector Array
		wgSafeArray(	VARTYPE  vt,			///< The base type of the array (the VARTYPE of each element of the array). The VARTYPE is restricted to a subset of the variant types. Neither the VT_ARRAY nor the VT_BYREF flag can be set. VT_EMPTY and VT_NULL are not valid base types for the array. All other types are legal. 
						unsigned int uintRows	///< Number of rows in the array. The number cannot be changed after the array is created. 
					);


			/// Constructor: Two Dimension Vector Array
		wgSafeArray(	VARTYPE  vt,			///< The base type of the array (the VARTYPE of each element of the array). The VARTYPE is restricted to a subset of the variant types. Neither the VT_ARRAY nor the VT_BYREF flag can be set. VT_EMPTY and VT_NULL are not valid base types for the array. All other types are legal. 
						unsigned int uintRows,	///< Number of rows in the array. The number cannot be changed after the array is created. 
						unsigned int uintColumns///< Number of columns in the array. The number cannot be changed after the array is created. 
					);


			/// Constructor: Three Dimension Vector Array
		wgSafeArray(	VARTYPE  vt, ///< The base type of the array (the VARTYPE of each element of the array). The VARTYPE is restricted to a subset of the variant types. Neither the VT_ARRAY nor the VT_BYREF flag can be set. VT_EMPTY and VT_NULL are not valid base types for the array. All other types are legal. 
						unsigned int uintRows,		///< Size of first dimension
						unsigned int uintColumns,	///< Size of second dimension
						unsigned int uintOutPage	///< Size of third dimension
					);


			/// Constructor: Four Dimension Vector Array
		wgSafeArray(	VARTYPE  vt, ///< The base type of the array (the VARTYPE of each element of the array). The VARTYPE is restricted to a subset of the variant types. Neither the VT_ARRAY nor the VT_BYREF flag can be set. VT_EMPTY and VT_NULL are not valid base types for the array. All other types are legal. 
						unsigned int uintRows,			///< Size of first dimension
						unsigned int uintColumns,		///< Size of second dimension
						unsigned int uintOutPage,		///< Size of third dimension
						unsigned int uintFourthDimension///< Size of fourth dimension
					);


			/// Destructor
		~wgSafeArray();


			/// provides the functionality of a copy constructor
			/// but enforces that the wgSafeArray is created first by the user.
			/// This means that the memory is not ambiguously created.
			/// The actual array must be allocated by the user before the copy.
			/// This function will copy as much as possible into the existing memory.
			/// \param other The array being copied into this location
		void SetArray(const VARIANTARG* other) throw();
		
			/// Initialise the internal variables, ready for use.<br>
			/// Example use: <br>
			/// <br>
			/// obj.Initialise(VT_VARIANT, 5,4,3);  to specify a 3 dimensional array of 5 rows 4 columns 3 outpage blocks<br><br>
			/// or obj.Initialise(VT_VARIANT, 5,4); to specify a 2 dimensional array of 5 rows and 4 columns only.<br>
		void Initialise(VARTYPE  vt,			///< The base type of the array (the VARTYPE of each element of the array). The VARTYPE is restricted to a subset of the variant types. Neither the VT_ARRAY nor the VT_BYREF flag can be set. VT_EMPTY and VT_NULL are not valid base types for the array. All other types are legal. 
						unsigned int uintDim1,///< Number of columns in the array. The number cannot be changed after the array is created. 
						...		///< More dimensions if required
			);

			/// Add an element of the type specified in Initialise at the position specified by the arguments
		void AddElement(void FAR* pv,				///< The data to be stored in the array
						unsigned int uintDim1,///< Number of Rows in the array. The number cannot be changed after the array is created. 
						...		///< More dimensions if required
			) throw();

			/// Get an element from the array. You must have memory enough to store it arranged at pv.
		void GetElement(void FAR* pv,			///< The data to be retreived from the array
						unsigned int uintDim1,///< Number of Rows in the array. The number cannot be changed after the array is created. 
						...		///< More dimensions if required
			) const throw();

			/// GetElement that returns a value converted to a string
		std::string GetElementAsString(unsigned int uintDim1, ...) const throw();

			/// GetNumberOfDimensions Does the equivalent of ::SafeArrayGetDim for this SafeArray
		int GetNumberOfDimensions() const throw();

			/// GetDimensionSizes retrieves the sizes of all the dimensions You must pass this 
			/// function an unsigned int array large enough to hold the information.
			/// \param puintDimSizes is a pointer to an array to hold the size of each dimension.
			/// Use GetNumberOfDimensions to find the number of dimensions.
		void GetDimensionSizes(unsigned int puintDimSizes[]) const throw();

			/// This function is only useful if you have an extra handle to the SAFEARRAY and 
			/// only want to use that without this object deleting it when it reaches the end of it's scope, 
			/// or when the delete on this object is called. 
		void ZeroWithoutRelease();

			/// Returns the member parray.
		SAFEARRAY* GetSafeArrayHandle();

	// Bunch of static helper functions for creating variants

		static void ClearVariant(VARIANTARG *pvarg);

			/// Clears a particular variant structure and releases any external objects
			/// or memory contained in the variant.  Supports the data types listed above.
		static void ReleaseVariant(VARIANTARG *pvarg);

		static VARIANTARG* SetVariantDispatch(VARIANTARG &Var ,IDispatch* pdisp);
		static VARIANTARG* SetVariantInt2(VARIANTARG &Var, int i);
		static VARIANTARG* SetVariantInt4(VARIANTARG &Var, int i);
		static VARIANTARG* SetVariantBool(VARIANTARG &Var, BOOL	b);
		static VARIANTARG* SetVariantDouble(VARIANTARG &Var, double	d);

			/// create an variant with an error code - for skipping optional arguments in 
			/// Invoke, use this function with error code DISP_E_PARAMNOTFOUND
		static VARIANTARG* SetVariantError(VARIANTARG &Var,	SCODE enuError);

		static VARIANTARG* SetVariantVariant(VARIANTARG &Var, VARIANT& lpsz);

			/// OLE and IDispatch use a BSTR as the representation of strings.
			/// This constructor automatically copies the passed-in C-style string
			/// into a BSTR.  It is important to not set the NOFREEVARIANT flag
			/// for this function, otherwise the allocated BSTR copy will probably
			/// get lost and cause a memory leak.
		static VARIANTARG* SetVariantCString(VARIANTARG &Var, const char* astring);

//		static VARIANTARG* SetVariantXloperString(VARIANTARG &Var, LPSTR astring);

		static VARIANTARG* SetVariantBSTR(VARIANTARG &Var, BSTR bstr);

			/// convert a BSTR into a string. A BSTR is a wide-character, length-prefixed string
		static std::string BSTRToString(BSTR strOriginal);

		static void WideCharacterString(wchar_t strBuffer[], int intBufferSize, const char* pcharValue);
		  // conver the specified value into a wide character string and return it in the  buffer


	private:
		
};



#endif // _WG_SAFE_ARRAY  

