#include "winutils/wuTCPSocket.h"


#ifndef _WU_MANAGED_TCP_SOCKET
#define _WU_MANAGED_TCP_SOCKET



/// ManagedPortFlagType is used to set the management flags for the wuManagedTCPSocket, which controls
/// how the port cycle is used during TryReconnect();
/// A combination of both flags means to first cycle through the specified ports, then try the any port... and repeat.
typedef enum
{
	ManagedCycle = 1,		///< ManagedCycle flag specifies to the wuManagedTCPSocket to cycle through the specified local port range. This repeats continuously.
	ManagedAnyPort   = 2	///< ManagedAnyPort flag specifies to the wuManagedTCPSocket to allow the binding of a non-specified port.

}ManagedPortFlagType;


/// wuManagedTCPSocket provides an extension on wuSocket which controls retries, and port ranges. 
/// This is useful in the case when only a select number of local ports 
/// may be bound to. 
class wuManagedTCPSocket : public wuTCPSocket
{
	public:

			/// Constructor. Set the Port range through variables usintLocalPortRangeStart and usintLocalPortRangeEnd.
			/// This range is ignored if _ManagementFlag does not have the ManagedCycle bit set. 
			/// If the management flag needs to be changed, the function SetManagementFlags() can be used. 
		wuManagedTCPSocket(	unsigned short	usintLocalPortRangeStart,
								unsigned short	usintLocalPortRangeEnd,
								ManagedPortFlagType _ManagementFlag
							);

			/// Constructor. Set the Port range through variables usintLocalPortRangeStart and usintLocalPortRangeEnd.
			/// This range is ignored if _ManagementFlag does not have the ManagedCycle bit set. 
			/// If the management flag needs to be changed, the function SetManagementFlags() can be used. 
		wuManagedTCPSocket(unsigned short	usintLocalPortRangeStart,
							unsigned short	usintLocalPortRangeEnd,
							unsigned short  usintRetries,
							ManagedPortFlagType _ManagementFlag
							);

			/// ManagedConnect should be used as a client connect to a server or listening port.
			/// This function accepts I.P address OR Host-Name.
			/// This function will connect using the next specified local port available, dependant on 
			/// the control flags set in the constructor.
			/// This function will close the existing Socket before opening another.
		virtual void ManagedConnect(const char* ahost, unsigned int intRemotePort, unsigned int uintReceiveTimeout) throw (wuSocketException);

			/// ManagedConnect should be used as a client connect to a server or listening port.
			/// This function accepts I.P address OR Host-Name.
			/// This function will connect using the next specified local port available, dependant on 
			/// the control flags set in the constructor.
			/// This function will close the existing Socket before opening another.
			/// NOTE!: The number of retries will be set depending on the Port range specified in this function.
			/// Any previously set number of retries will be overwritten. 		
		virtual void ManagedConnect(const char* ahost, 
									unsigned short	usintLocalPortRangeStart, 
									unsigned short usintLocalPortRangeEnd,
									unsigned int intRemotePort, 
									unsigned int uintReceiveTimeout
									) throw (wuSocketException);


			/// Get the Management flag bits currently used with this wuManagedTCPSocket.
		inline ManagedPortFlagType GetManagementFlags(){return (ManagedPortFlagType)mintManagementFlag;}


			/// Set the Management flag bits to be used with this wuManagedTCPSocket.
		inline void SetManagementFlags(const ManagedPortFlagType& Flags){mintManagementFlag = (int)Flags;}


			/// Send will try to send buf, to the size of len. If it fails, it will try to reconnect 
			/// and do so usintRetries number of times. If failures occur due to local port in use error, 
			/// the socket will automatically be tried on another port.  
			/// This overload of Send sets the flags used in the win32 level library to zero. 
			/// Please see the MSDN for more detail.
		virtual int Send(const char FAR* buf, int len);	

			/// Send will try to send buf, to the size of len. If it fails, it will try to reconnect 
			/// and do so usintRetries number of times. If failures occur due to local port in use error, 
			/// the socket will automatically be tried on another port. 
			/// This overload of Send allows the settings of the flags argument, of which there are few possible
			/// values: MSG_DONTROUTE, and MSG_OOB. Please see the MSDN for more detail.
		virtual int Send(const char FAR* buf, int len, int flags);

			/// move the Port counter to the next port. This port number depends on the ManagementFlag 
			/// which sets the port finding strategy. 
		void IncrementCycleStage();

			/// Try to Force a reconnection. If this fails for reason of a local port in use, the port number will
			/// be automatically changed for the next try. TryReconnect will NOT however try to reconnect 
			/// more than once.
		bool TryReconnect();


	protected:

		void Settings(	unsigned short	usintLocalPortRangeStart, 
						unsigned short usintLocalPortRangeEnd, 
						unsigned short  usintRetries, 
						ManagedPortFlagType _ManagementFlag);

		void Settings(	unsigned short	usintLocalPortRangeStart, 
						unsigned short usintLocalPortRangeEnd, 
						ManagedPortFlagType _ManagementFlag);

		void SetPortRange(	unsigned short	usintLocalPortRangeStart, 
							unsigned short usintLocalPortRangeEnd);

		unsigned short musintLocalPortRangeStart;
		unsigned short musintLocalPortRangeEnd;
		unsigned short musintRetries;
		int mintManagementFlag;

	private:

		int ReConnectAndSend(const char FAR* buf, int len, int flags, int intRecursiveSendTry);
		bool CycleManagedReConnect(int intSendReTry);

};


#endif//_WU_MANAGED_TCP_SOCKET

